# -*- coding: utf-8 -*-
sectors = {
    "Agriculture": [
        "be/agriculture/agriculture",
        "be/landbouw/landbouw-1",
        "gr/georgia/georgia",
        "es/agricultura/agricultura-c",
        "es/agricultura/agricultura-e",
        "is/agriculture/agriculture",
        "pt/agricultura/agricultura",
        "lt/lt-test-sector/zemes-ukis",
        "lv/latvia-test/lauksaimnieciba",
        "eu/eu-agriculture/oira-agriculture",
    ],
    "COVID": [
        "si/covid-19/covid-19",
        "hr/covid-19/covid-19",
        "gr/covid-19/covid-19",
        "be/covid-19/covid-19-fr",
        "be/covid-19/covid-19-nl",
        "cz/covid-19/covid-19-cz",
        "lv/latvia-test/covid-19",
        "lt/covid-19/covid-19",
        "bg/covid-19/covid-19",
        "fr/covid/covid",
        "pt/covid-19/covid-19",
        "cy/covid-19/covid-19",
        "es/covid/covid",
        "es/covid/covid-19-es",
        "eu/covid-19/covid-19",
    ],
    "Cleaning": [
        "cz/uklidovu-pracovnik/uklidovu-pracovnik",
        "eu/eu-cleaning/eu-cleaning",
        "be/schoonmaak/schoonmaak",
        "be/schoonmaak/nettoyage-classique",
        "lt/lt-test-sector/misko-ruosa",
        "si/ciscenje-poslovnih-prostorov/ciscenje-poslovnih-prostorov-1",
        "lv/latvia-test/uzkopsanas-darbi",
    ],
    "Education": [
        "es/educacion/secundaria",
        "es/educacio/secundaria",
        "es/educacio/preescolar",
        "es/educacion/preescolar",
        "hr/njegovatelji-djece/njegovatelji-djece-m1",
        "si/ucitelj/ucitelj",
        "eu/eu-education/oira-early-childhood-education-and-care",
        "eu/eu-education/oira-secondary-education",
        "gr/ekpaideutikes-monades/educational-premises-gr",
        "lt/lt-test-sector/svietimo-istaigos",
        "es/autoescuelas/autoescuelas",
        "lv/latvia-test/izglitibas-iestades-1",
        "cy/schools2015/scholikes-monades",
        "bg/obrazovanie-1/obrazovanie",
    ],
    "Hairdressers": [
        "fr/soins-esthetiques/soins-esthetiques",
        "fr/coiffure/coiffure",
        "lt/lt-test-sector/kirpyklos-ir-grozio-salonai",
        "cy/hairdressers/koureia-kommoteria-cy-hairdressing_v2",
        "hr/frizerski-salon/frizerski-salon-1",
        "eu/eu-hairdressing/eu-hairdressing",
        "is/icelandic-sector-leifur/harsnyrtistofur-1-desember-2016",
        "es/perruqueries/peluquerias",
        "gr/hairdressers/hairdressing-salons",
        "cz/kadernicke-sluzby/kadernicke-sluzby",
        "pt/cabeleireiro/cabeleireirofinal",
        "be/coiffure/oira-coiffure",
        "be/coiffure/oira-kappers",
        "si/frizer/delo-v-frizerski-dejavnosti",
    ],
    "HORECA": [
        "be/oira-horeca/oira-horeca",
        "be/oira-horeca/oira-horeca-test-nl",
        "bg/khotieli-i-riestoranti/khotieli-i-riestoranti",
        "cy/etaireies-trophodosias-catering/klados-etaireion-trophodosias-catering",
        "cy/kaphestiatoria-1/kaphestiatoria",
        "cy/mikra-touristika-katalumata/mikra-touristika-katalumata",
        "es/horeca/horeca",
        "es/horeca/horeca-1",
        "eu/eu-horeca/oira-horeca",
        "fr/hotellerie-restauration/hcr",
        "fr/restauration-rapide/restauration-rapide/",
        "fr/traiteurs-et-organisateurs-de-reception/traiteurs-et-organisateurs-de-reception/",
        "gr/catering/ergaleio-oira-gia-tin-ektimisi-kindynoy-ston-klado-ton-etaireion-trofodosias-catering",
        "gr/kapheteries-kaphe-estiatoria/kapheteries-kaphe-estiatoria-ekd-3e-iounios-2022",
        "hr/konobar/konobar",
        "hr/kuhar/kuharica",
        "hr/radnik-u-cateringu/radnik-u-cateringu",
        "is/catering/veitingar-sept-2015",
        "lt/lt-test-sector/viesbuciai-ir-restoranai",
        "lv/latvia-test/sabiedriska-edinasana",
        "lv/latvia-test/viesnicas-un-viesu-majas",
        "mt/catering/catering-mt",
        "pt/restauracao/restauracao",
        "si/kuhar/kuhar",
        "si/natakar/natakar",
    ],
    "Leather and tanning": [
        "lv/eu/leather-tanning/leather-and-tanning-version-2",
        "pt/curtumes/curtumes",
        "es/adoberies/curtiduria",
        "es/adoberies/adoberies",
        "bg/obrabotka-na-kozhi-proizvodstvo-na-obuvki/obrabotka-na-kozhi-proizvodstvo-na-obuvki",
        "eu/leather-tanning/leather-tanning-draft-risk-assessment-tool",
    ],
    "Live performance": [
        "fi/esittava-taide/esittava-taide",
        "gr/epikheireseis-khoron-parousiases-zontanon-ekdeloseon/epikheireseis-khoron-parousiases-zontanon-ekdeloseon",
        "gr/epikheireseis-paragoges-zontanon-ekdeloseon-1/epikheireseis-khoron-parousiases-zontanon-ekdeloseon",
        "be/podiumkunsten-locaties-nl-preview/podiumkunsten-locaties-nl-preview",
        "be/podia/podiumkunsten-fr-preview-08012018",
        "be/podiumkunsten/oira-podiumkunsten",
        "be/podiumkunsten-locaties-fr-preview/podiumkunsten-locaties-fr-preview",
        "eu/live-performance/eu-representaciones-en-vivo-recintos-y-salas",
        "eu/live-performance/ue-representaciones-en-vivo-producciones",
        "eu/live-performance/live-performance-oira-tool_venues",
        "eu/live-performance/live-performance-oira-tool_productions",
        "is/live-performance/lifandi-flutningur-svidslistir",
    ],
    "Private security": [
        "es/seguretatprivada/seguridad-privada",
        "gr/private-security-sector/etaireies-parokhes-uperesion-asphaleias",
        "es/seguretatprivada/seguretat-privada",
        "eu/eu-private-security/private-security-eu",
    ],
    "Sports": [
        "lv/latvia-test/sporta-centrs",
        "eu/eu-sport-sector/eu-sport-sector-active-leisure",
        "eu/eu-sport-sector/eu-sport-sector-not-for-profit",
        "eu/eu-sport-sector/eu-sport-sector-professional-sports",
    ],
}
